@@ -76,9 +76,11 @@ const addPromiseAPI = function(Modbus) {
     cl.readCoils = _convert(cl.writeFC1);
     cl.readDiscreteInputs = _convert(cl.writeFC2);
     cl.readHoldingRegisters = _convert(cl.writeFC3);
+    cl.readRegistersEnron = _convert(cl.writeFC3);
     cl.readInputRegisters = _convert(cl.writeFC4);
     cl.writeCoil = _convert(cl.writeFC5);
     cl.writeRegister = _convert(cl.writeFC6);
+    cl.writeRegisterEnron = _convert(cl.writeFC6);
     cl.writeCoils = _convert(cl.writeFC15);
     cl.writeRegisters = _convert(cl.writeFC16);
     cl.readFileRecords = _convert(cl.writeFC20);
